<?php
/**
 * @file
 * Classes and related functions for survey and survey response entity types
 */

/**
 * Main class for survey entities.
 */
class SurveyEntity extends SurveyBuilderEntity {
  protected $typeName = 'survey';
}

/**
 * Load a single survey entity from its ID
 *
 * @param $pid
 *   The entity ID.
 */
function survey_load($pid) {
  $entities = entity_load('survey', array($pid));
  if (count($entities)) {
    return array_shift($entities);
  }
  return FALSE;
}

/**
 * Delete a single survey entity from its ID
 *
 * @param $pid
 *   The entity ID.
 */
function survey_delete($pid) {
  return entity_delete('survey', $pid);
}

/**
 * Main class for survey response entities.
 */
class SurveyResponseEntity extends SurveyBuilderEntity {
  protected $typeName = 'survey_response';
}

/**
 * Metadata class for survey response entities
 */
class SurveyResponseEntityMetadata extends SurveyBuilderEntityMetadata {

  public function entityPropertyInfo() {
    $info = parent::entityPropertyInfo();
    $properties =& $info[$this->type]['properties'];
    $properties['survey_id']['label'] = t('Survey');
    $properties['survey_id']['type'] = 'survey';
    return $info;
  }

}

/**
 * Load a single survey response entity from its ID
 *
 * @param $pid
 *   The entity ID.
 */
function survey_response_load($pid) {
  $entities = entity_load('survey_response', array($pid));
  if (count($entities)) {
    return array_shift($entities);
  }
  return FALSE;
}

/**
 * Save survey response
 */
function survey_response_save($survey_id, $values, $form_state) {

  global $user;

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.question');

  $survey = survey_load($survey_id);
  if (!$survey) {
    watchdog('survey_builder', 'Invalid survey: %id', array('%id' => $survey_id), WATCHDOG_ERROR);
    return FALSE;
  }

  $response = new SurveyResponseEntity();
  $response->survey_id = $survey_id;
  $response->uid = $user->uid;
  $response->created = time();

  // Allow other modules to alter survey responses
  drupal_alter('survey_builder_survey_response_save', $response, $survey, $values, $form_state);

  $response->save();

  $success = survey_response_questions_save($survey->fields, $values, $response->pid);
  if (!$success) {
    return FALSE;
  }

  module_invoke_all('survey_builder_survey_response_saved', $response, $survey, $values, $form_state);

  return $response->pid;

}

function survey_response_questions_save($questions, $values, $survey_response_id) {

  foreach ($questions as $key => $field) {

    if (strpos($key, '#') === 0) {
      continue;
    }
    elseif (isset($field['#type']) && $field['#type'] == 'fieldset') {

      $success = survey_response_questions_save($field, $values[$key], $survey_response_id);
      if (!$success) {
        return FALSE;
      }

    }
    else {

      if (!isset($values[$key])) {
        $values[$key] = '';
      }

      $success = question_response_save($survey_response_id, $field['#question_id'], $key, $values[$key]);
      if (!$success) {
        watchdog('survey_builder', 'Unable to save response for question: %id',
          array('%id' => $field['#question_id']), WATCHDOG_ERROR);
        return FALSE;
      }

    }

  }

  return TRUE;

}

/**
 * Survey response validate
 */
function survey_response_validate($survey_id, $values, $form_state) {

  $survey = survey_load($survey_id);
  if (!$survey) {
    watchdog('survey_builder', 'Invalid survey: %id', array('%id' => $survey_id), WATCHDOG_ERROR);
    return FALSE;
  }

  module_load_include('inc', 'survey_builder', 'includes/survey_builder.admin');

  $validations = module_invoke_all('survey_builder_survey_response_validate', $survey, $values, $form_state);
  $validations[] = survey_response_questions_validate($survey->fields, $values);

  foreach($validations as $pass) {
      if (!$pass) {
          return false;
      }
  }

  return true;

}

function survey_response_questions_validate($questions, $values) {

  $success = true;

  foreach ($questions as $key => $field) {

    if (strpos($key, '#') === 0) {
      continue;

    } elseif (isset($field['#type']) && $field['#type'] == 'fieldset') {

      $ret = survey_response_questions_validate($field, $values);
      if (!$ret) {
        $success = false;
      }

    } else {

      $question = question_load($field['#question_id']);
      if (!$question) {
        watchdog('survey_builder', 'Unable to load question for validation: %id',
          array('%id' => $field['#question_id']), WATCHDOG_ERROR);
        continue;

      } else if (!isset($values[$key])) {
        $values[$key] = '';

      }

      $ret = question_field_response_validate($question->field, $values[$key]);
      if (!$ret) {
        form_set_error($key, t('The !name field is required', array('!name' => $field['#title'])));
        $success = false;
      }

    }

  }

  return $success;

}
